@@ -300,6 +300,13 @@ phoc_output_init (PhocOutput *self)
 
   wl_list_init (&self->layer_surfaces);
 
+  wl_list_init (&priv->damage.link);
+  wl_list_init (&priv->frame.link);
+  wl_list_init (&priv->needs_frame.link);
+  wl_list_init (&priv->request_state.link);
+  wl_list_init (&self->commit.link);
+  wl_list_init (&self->output_destroy.link);
+
   priv->scale_filter = PHOC_OUTPUT_SCALE_FILTER_AUTO;
 
   priv->renderer = g_object_ref (phoc_server_get_renderer (server));
@@ -569,7 +576,6 @@ phoc_output_draw (PhocOutput *self)
   struct wlr_output *wlr_output = self->wlr_output;
   bool needs_frame, scanned_out = false;
   pixman_region32_t buffer_damage, frame_damage;
-  int buffer_age;
   PhocRenderContext render_context;
   struct wlr_buffer *buffer;
   struct wlr_render_pass *render_pass;
@@ -603,7 +609,7 @@ phoc_output_draw (PhocOutput *self)
   if (!wlr_output_configure_primary_swapchain (wlr_output, &pending, &wlr_output->swapchain))
     goto out;
 
-  buffer = wlr_swapchain_acquire (wlr_output->swapchain, &buffer_age);
+  buffer = wlr_swapchain_acquire (wlr_output->swapchain, NULL);
   if (!buffer)
     goto out;
 
@@ -614,7 +620,7 @@ phoc_output_draw (PhocOutput *self)
   }
 
   pixman_region32_init (&buffer_damage);
-  wlr_damage_ring_get_buffer_damage (&self->damage_ring, buffer_age, &buffer_damage);
+  wlr_damage_ring_rotate_buffer (&self->damage_ring, buffer, &buffer_damage);
 
   render_context = (PhocRenderContext){
     .output = self,
@@ -639,8 +645,6 @@ phoc_output_draw (PhocOutput *self)
   if (!wlr_output_commit_state (wlr_output, &pending))
     goto out;
 
-  wlr_damage_ring_rotate (&self->damage_ring);
-
  out:
   wlr_output_state_finish (&pending);
 }
@@ -736,7 +740,7 @@ phoc_output_handle_commit (struct wl_listener *listener, void *data)
     if (configure_sent &&
         !priv->modeset_shield &&
         !phoc_output_shield_is_raised (priv->shield)) {
-      phoc_output_raise_shield (self);
+      phoc_output_raise_shield (self, FALSE);
       priv->modeset_shield = TRUE;
     }
   }
@@ -1307,7 +1311,7 @@ phoc_output_xwayland_children_for_each_surface (PhocOutput                  *sel
  * @iterator: (scope call): The callback invoked on each iteration
  * @user_data: Callback user data
  *
- * Iterate over a [type@LayerSurface] and it's popups.
+ * Iterate over a [type@LayerSurface] and its popups.
  */
 void
 phoc_output_layer_surface_for_each_surface (PhocOutput          *self,
@@ -1316,27 +1320,18 @@ phoc_output_layer_surface_for_each_surface (PhocOutput          *self,
                                             void                *user_data)
 {
   struct wlr_layer_surface_v1 *wlr_layer_surface_v1 = layer_surface->layer_surface;
+  PhocOutputSurfaceIteratorData data = {
+    .user_iterator = iterator,
+    .user_data = user_data,
+    .output = self,
+    .ox = layer_surface->geo.x,
+    .oy = layer_surface->geo.y,
+    .scale = 1.0,
+  };
 
-  phoc_output_surface_for_each_surface (self, wlr_layer_surface_v1->surface,
-                                        layer_surface->geo.x,
-                                        layer_surface->geo.y, iterator,
-                                        user_data);
-
-  struct wlr_xdg_popup *state;
-  wl_list_for_each (state, &wlr_layer_surface_v1->popups, link) {
-    struct wlr_xdg_surface *popup = state->base;
-    if (!popup->configured)
-      continue;
-
-    double popup_sx, popup_sy;
-    popup_sx = layer_surface->geo.x;
-    popup_sx += popup->popup->current.geometry.x - popup->current.geometry.x;
-    popup_sy = layer_surface->geo.y;
-    popup_sy += popup->popup->current.geometry.y - popup->current.geometry.y;
-
-    phoc_output_xdg_surface_for_each_surface (self, popup,
-                                              popup_sx, popup_sy, iterator, user_data);
-  }
+  wlr_layer_surface_v1_for_each_surface (wlr_layer_surface_v1,
+                                         phoc_output_for_each_surface_iterator,
+                                         &data);
 }
 
 /**
@@ -1679,21 +1674,18 @@ damage_whole_view (PhocOutput *self, PhocView  *view)
   }
 }
 
-
 /**
  * phoc_output_damage_from_view:
  * @self: The output to add damage to
  * @view: The view providing the damage
- * @whole: Whether
+ * @whole: Whether to damage the whole view
  *
  * Adds a [type@PhocView]'s damage to the damaged area of @self. If
- * @whole is %TRUE the whole view is damaged (including any window
- * decorations if they exist). If @whole is %FALSE only buffer damage
- * is taken into account.
- * Also schedules a new frame.
+ * `whole` is `TRUE` the whole surface area is explicitly damaged.
+ * Otherwise only already present damage is collected.
  */
 void
-phoc_output_damage_from_view (PhocOutput *self, PhocView  *view, bool whole)
+phoc_output_damage_from_view (PhocOutput *self, PhocView *view, bool whole)
 {
   if (!phoc_view_accept_damage (self, view)) {
     return;
@@ -1705,8 +1697,34 @@ phoc_output_damage_from_view (PhocOutput *self, PhocView  *view, bool whole)
   phoc_output_view_for_each_surface (self, view, damage_surface_iterator, &whole);
 }
 
+/**
+ * phoc_output_damage_from_layer_surface:
+ * @self: The output to add damage to
+ * @layer_surface: The layer surface providing the damage
+ * @whole: Whether to damage the whole surface
+ *
+ * Adds a [type@PhocLayerSurface]'s damage to the damaged area of
+ * @self. If `whole` is `TRUE` the whole surface area is explicitly
+ * damaged. Otherwise only already present damage is collected.
+ */
 void
-phoc_output_damage_whole_drag_icon (PhocOutput *self, PhocDragIcon *icon)
+phoc_output_damage_from_layer_surface (PhocOutput       *self,
+                                       PhocLayerSurface *layer_surface,
+                                       gboolean          whole)
+{
+  phoc_output_layer_surface_for_each_surface (self, layer_surface, damage_surface_iterator, &whole);
+}
+
+/**
+ * phoc_output_damage_from_drag_icon:
+ * @self: The output to add damage to
+ * @icon: The drag icon providing the damage
+ *
+ * Adds a drag icon's damage to the damaged area of @self. We always
+ * damage the whole surface.
+ */
+void
+phoc_output_damage_from_drag_icon (PhocOutput *self, PhocDragIcon *icon)
 {
   bool whole = true;
 
@@ -1717,26 +1735,26 @@ phoc_output_damage_whole_drag_icon (PhocOutput *self, PhocDragIcon *icon)
                                         damage_surface_iterator, &whole);
 }
 
-void
-phoc_output_damage_whole_surface (PhocOutput         *self,
-                                  struct wlr_surface *surface,
-                                  double              ox,
-                                  double              oy)
-{
-  bool whole = true;
-
-  phoc_output_surface_for_each_surface (self, surface, ox, oy,
-                                        damage_surface_iterator, &whole);
-}
-
+/**
+ * phoc_output_damage_from_surface:
+ * @self: The output to add damage to
+ * @wlr_surface: The wlr_surface providing the damage
+ * @ox: x coordinate of the surface in output local coordinates
+ * @oy: y coordinate of the surface in output local coordinates
+ * @whole: Whether to damage the hole surface
+ *
+ * Adds a surface's damage to the damaged area of @self. If `whole`
+ * is `TRUE` the whole surface area is explicitly damaged. Otherwise
+ * only already present damage is collected.
+ */
 void
 phoc_output_damage_from_surface (PhocOutput         *self,
                                  struct wlr_surface *wlr_surface,
                                  double              ox,
-                                 double              oy)
-{
-  bool whole = false;
+                                 double              oy,
+                                 gboolean            whole)
 
+{
   phoc_output_surface_for_each_surface (self, wlr_surface, ox, oy,
                                         damage_surface_iterator, &whole);
 }
@@ -2091,12 +2109,13 @@ phoc_output_lower_shield (PhocOutput *self, PhocEasing easing, guint duration)
 /**
  * phoc_output_raise_shield:
  * @self: The output to raise the shield for
+ * @show_spinner: Whether to animate a spinner on the raised shield
  *
  * Raise an output shield will be put in place to hide the outputs
- * current content.
+ * current content. A spinner can be displayed on the raised shield.
  */
 void
-phoc_output_raise_shield (PhocOutput *self)
+phoc_output_raise_shield (PhocOutput *self, gboolean show_spinner)
 {
   PhocOutputPrivate *priv;
 
@@ -2106,7 +2125,7 @@ phoc_output_raise_shield (PhocOutput *self)
   if (priv->shield == NULL)
     priv->shield = phoc_output_shield_new (self);
 
-  phoc_output_shield_raise (priv->shield);
+  phoc_output_shield_raise (priv->shield, show_spinner);
 }
 
 /**
