@@ -36,8 +36,9 @@
 #include <wlr/render/wlr_renderer.h>
 #include <wlr/render/gles2.h>
 #include <wlr/render/egl.h>
-#include <wlr/types/wlr_compositor.h>
+#include <wlr/types/wlr_alpha_modifier_v1.h>
 #include <wlr/types/wlr_buffer.h>
+#include <wlr/types/wlr_compositor.h>
 #include <wlr/types/wlr_linux_dmabuf_v1.h>
 #include <wlr/util/region.h>
 #include <wlr/util/transform.h>
@@ -130,7 +131,7 @@ phoc_renderer_get_property (GObject    *object,
 static void
 render_texture (PhocOutput               *output,
                 struct wlr_texture       *texture,
-                const struct wlr_fbox    *_src_box,
+                const struct wlr_fbox    *src_box,
                 const struct wlr_box     *dst_box,
                 const struct wlr_box     *clip_box,
                 enum wl_output_transform  surface_transform,
@@ -139,22 +140,21 @@ render_texture (PhocOutput               *output,
 {
   pixman_region32_t damage;
   struct wlr_box proj_box = *dst_box;
-  struct wlr_fbox src_box = {0};
   enum wl_output_transform transform;
 
+  if (alpha == 0.0)
+    return;
+
   if (!phoc_utils_is_damaged (&proj_box, ctx->damage, clip_box, &damage))
     goto buffer_damage_finish;
 
-  if (_src_box)
-    src_box = *_src_box;
-
   phoc_output_transform_box (output, &proj_box);
   phoc_output_transform_damage (output, &damage);
   transform = wlr_output_transform_compose (surface_transform, output->wlr_output->transform);
 
   wlr_render_pass_add_texture (ctx->render_pass, &(struct wlr_render_texture_options) {
       .texture = texture,
-      .src_box = src_box,
+      .src_box = *src_box,
       .dst_box = proj_box,
       .transform = transform,
       .alpha = &alpha,
@@ -177,6 +177,7 @@ render_surface_iterator (PhocOutput         *output,
   PhocRenderContext *ctx = data;
   struct wlr_output *wlr_output = output->wlr_output;
   float alpha = ctx->alpha;
+  const struct wlr_alpha_modifier_surface_v1_state *alpha_modifier_state;
 
   struct wlr_texture *texture = wlr_surface_get_texture (surface);
   if (!texture)
@@ -193,7 +194,18 @@ render_surface_iterator (PhocOutput         *output,
   phoc_utils_scale_box (&clip_box, scale);
   phoc_utils_scale_box (&clip_box, wlr_output->scale);
 
-  render_texture (output, texture, &src_box, &dst_box, &clip_box, surface->current.transform, alpha, ctx);
+  alpha_modifier_state = wlr_alpha_modifier_v1_get_surface_state (surface);
+  if (alpha_modifier_state)
+    alpha *= (float)alpha_modifier_state->multiplier;
+
+  render_texture (output,
+                  texture,
+                  &src_box,
+                  &dst_box,
+                  &clip_box,
+                  surface->current.transform,
+                  alpha,
+                  ctx);
 
   wlr_presentation_surface_scanned_out_on_output (surface, wlr_output);
 }
@@ -301,33 +313,17 @@ view_render_to_buffer_iterator (struct wlr_surface *surface, int sx, int sy, voi
   phoc_view_get_geometry (data->view, &geo);
   wlr_surface_get_buffer_source_box (surface, &src_box);
 
-  float src_scale = MIN (src_box.width / surface->current.width,
-                         src_box.height / surface->current.height);
-  phoc_utils_scale_box (&geo, src_scale);
-
-  /* Geometry needs to be fully within the surface so we don't sample
-   * out of bounds. For other cases (e.g. the geometry being extended
-   * due to subsurfaces) just use the surface verbatim */
-  if (geo.x > src_box.x && geo.y > src_box.y &&
-      geo.width < src_box.width && geo.height < src_box.height) {
-    src_box = (struct wlr_fbox) {
-      .x = geo.x,
-      .y = geo.y,
-      .width = geo.width,
-      .height = geo.height,
-    };
-  }
-
-  float scale = fmin (data->width / (float)src_box.width,
-                      data->height / (float)src_box.height);
-
   struct wlr_box dst_box = {
-    .x = sx * scale,
-    .y = sy * scale,
-    .width = src_box.width * scale,
-    .height = src_box.height * scale,
+    .x = -geo.x + sx,
+    .y = -geo.y + sy,
+    .width = surface->current.width,
+    .height = surface->current.height,
   };
 
+  float scale = fmin (data->width / (float)geo.width,
+                      data->height / (float)geo.height);
+  phoc_utils_scale_box (&dst_box, scale);
+
   wlr_render_pass_add_texture (data->render_pass, &(struct wlr_render_texture_options) {
       .texture = texture,
       .src_box = src_box,
