@@ -56,10 +56,15 @@ gboolean               phoc_server_setup                   (PhocServer *self,
                                                             PhocConfig *config,
                                                             const char *exec,
                                                             GMainLoop  *mainloop,
-                                                            PhocServerFlags flags,
-                                                            PhocServerDebugFlags debug_flags);
+                                                            PhocServerFlags flags);
+void                   phoc_server_set_debug_flags         (PhocServer *self,
+                                                            PhocServerDebugFlags flags);
+PhocServerDebugFlags   phoc_server_get_debug_flags         (PhocServer *self);
 gboolean               phoc_server_check_debug_flags       (PhocServer *self,
                                                             PhocServerDebugFlags check);
+void                   phoc_server_set_log_domains         (PhocServer *self,
+                                                            const char * const *log_domains);
+const char *const *    phoc_server_get_log_domains         (PhocServer *self);
 const char            *phoc_server_get_session_exec        (PhocServer *self);
 gint                   phoc_server_get_session_exit_status (PhocServer *self);
 PhocRenderer          *phoc_server_get_renderer            (PhocServer *self);
