@@ -79,17 +79,149 @@ typedef struct _PhocViewPrivate {
   struct wl_listener toplevel_handle_request_activate;
   struct wl_listener toplevel_handle_request_fullscreen;
   struct wl_listener toplevel_handle_request_close;
+  /* ext-foreign-toplevel-list */
+  struct wlr_ext_foreign_toplevel_handle_v1 *ext_foreign_toplevel_v1_handle;
 
   /* Subsurface and popups */
   struct wl_listener surface_new_subsurface;
   GSList            *child_surfaces;
 } PhocViewPrivate;
 
-G_DEFINE_TYPE_WITH_PRIVATE (PhocView, phoc_view, G_TYPE_OBJECT)
+static void phoc_view_child_root_iface_init (PhocChildRootInterface *iface);
+
+G_DEFINE_TYPE_WITH_CODE (PhocView, phoc_view, G_TYPE_OBJECT,
+                         G_ADD_PRIVATE (PhocView)
+                         G_IMPLEMENT_INTERFACE (PHOC_TYPE_CHILD_ROOT,
+                                                phoc_view_child_root_iface_init))
+
+
 #define PHOC_VIEW_SELF(p) PHOC_PRIV_CONTAINER(PHOC_VIEW, PhocView, (p))
 
 static bool view_center (PhocView *view, PhocOutput *output);
 
+/* {{{ PhocChildRoot interface */
+
+static void
+phoc_view_child_root_get_box (PhocChildRoot *root, struct wlr_box *box)
+{
+  PhocView *self = PHOC_VIEW (root);
+
+  g_assert (PHOC_IS_VIEW (self));
+
+  phoc_view_get_box (self, box);
+}
+
+
+static gboolean
+phoc_view_child_root_is_mapped (PhocChildRoot *root)
+{
+  PhocView *self = PHOC_VIEW (root);
+
+  g_assert (PHOC_IS_VIEW (self));
+
+  return !!phoc_view_is_mapped (self);
+}
+
+
+static void
+phoc_view_child_root_apply_damage (PhocChildRoot *root)
+{
+  PhocView *self = PHOC_VIEW (root);
+
+  g_assert (PHOC_IS_VIEW (self));
+
+  phoc_view_apply_damage (self);
+}
+
+
+static void
+phoc_view_child_root_add_child (PhocChildRoot *root, PhocViewChild *child)
+{
+  PhocView *self = PHOC_VIEW (root);
+  PhocViewPrivate *priv;
+
+  g_assert (PHOC_IS_VIEW (self));
+  g_assert (PHOC_IS_VIEW_CHILD (child));
+  priv = phoc_view_get_instance_private (self);
+
+  priv->child_surfaces = g_slist_prepend (priv->child_surfaces, child);
+}
+
+
+static void
+phoc_view_child_root_remove_child (PhocChildRoot *root, PhocViewChild *child)
+{
+  PhocView *self = PHOC_VIEW (root);
+  PhocViewPrivate *priv;
+
+  g_assert (PHOC_IS_VIEW (self));
+  g_assert (PHOC_IS_VIEW_CHILD (child));
+  priv = phoc_view_get_instance_private (self);
+
+  priv->child_surfaces = g_slist_remove (priv->child_surfaces, child);
+}
+
+
+static gboolean
+phoc_view_child_root_unconstrain_popup (PhocChildRoot *root, struct wlr_box *box)
+{
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
+  PhocView *self = PHOC_VIEW (root);
+  struct wlr_surface_output *surface_output;
+  struct wlr_box usable_area;
+  struct wlr_box geom;
+  PhocOutput *output;
+
+  g_assert (PHOC_IS_VIEW (self));
+
+  if (!phoc_view_is_mapped (self))
+    return FALSE;
+
+  /* Try top left corner of the view's geometry: */
+  phoc_view_get_geometry (self, &geom);
+  output = phoc_desktop_layout_get_output (desktop, self->box.x + geom.x, self->box.y + geom.y);
+  if (!output && wl_list_empty (&self->wlr_surface->current_outputs))
+    return FALSE;
+
+  /* Otherwise just take the first output */
+  surface_output = wl_container_of (self->wlr_surface->current_outputs.next, surface_output, link);
+  output = PHOC_OUTPUT (surface_output->output->data);
+  g_assert (PHOC_IS_OUTPUT (output));
+
+  if (!output) {
+    g_warning ("No output found for view %p at %d,%d", self, self->box.x, self->box.y);
+    return FALSE;
+  }
+
+  usable_area = output->usable_area;
+  usable_area.x += output->lx;
+  usable_area.y += output->ly;
+
+  /* the output box expressed in the coordinate system of the toplevel parent
+   * of the popup */
+  *box = (struct wlr_box) {
+    .x = usable_area.x - self->box.x,
+    .y = usable_area.y - self->box.y,
+    .width = usable_area.width,
+    .height = usable_area.height,
+  };
+
+  return TRUE;
+}
+
+
+static void
+phoc_view_child_root_iface_init (PhocChildRootInterface *iface)
+{
+  iface->get_box = phoc_view_child_root_get_box;
+  iface->is_mapped = phoc_view_child_root_is_mapped;
+  iface->apply_damage = phoc_view_child_root_apply_damage;
+  iface->add_child = phoc_view_child_root_add_child;
+  iface->remove_child = phoc_view_child_root_remove_child;
+  iface->unconstrain_popup = phoc_view_child_root_unconstrain_popup;
+}
+
+/* }}} */
 
 static void
 toggle_decoration (PhocView *self)
@@ -204,6 +336,17 @@ view_create_foreign_toplevel_handle (PhocView *self)
   wl_signal_add(&priv->toplevel_handle->events.request_close, &priv->toplevel_handle_request_close);
 
   priv->toplevel_handle->data = self;
+
+  wlr_foreign_toplevel_handle_v1_set_title (priv->toplevel_handle, priv->title ?: "");
+  wlr_foreign_toplevel_handle_v1_set_app_id (priv->toplevel_handle, priv->app_id ?: "");
+
+  struct wlr_ext_foreign_toplevel_handle_v1_state foreign_toplevel_state = {
+    .app_id = priv->app_id,
+    .title = priv->title,
+  };
+  priv->ext_foreign_toplevel_v1_handle =
+    wlr_ext_foreign_toplevel_handle_v1_create (desktop->ext_foreign_toplevel_list_v1,
+                                               &foreign_toplevel_state);
 }
 
 
@@ -219,6 +362,9 @@ phoc_view_destroy_toplevel_handle (PhocView *self)
   wl_list_remove (&priv->toplevel_handle_request_close.link);
   wlr_foreign_toplevel_handle_v1_destroy (priv->toplevel_handle);
   priv->toplevel_handle = NULL;
+
+  wlr_ext_foreign_toplevel_handle_v1_destroy (priv->ext_foreign_toplevel_v1_handle);
+  priv->ext_foreign_toplevel_v1_handle = NULL;
 }
 
 
@@ -788,9 +934,6 @@ phoc_view_set_fullscreen (PhocView *view, bool fullscreen, PhocOutput *output)
     if (was_fullscreen)
       priv->fullscreen_output->fullscreen_view = NULL;
 
-    struct wlr_box view_box;
-    phoc_view_get_box (view, &view_box);
-
     view_save (view);
 
     struct wlr_box output_box;
@@ -991,10 +1134,9 @@ view_center (PhocView *view, PhocOutput *output)
     struct wlr_output *wlr_output = wlr_output_layout_output_at (desktop->layout,
                                                                  cursor->cursor->x,
                                                                  cursor->cursor->y);
-    if (!wlr_output) {
-      // empty layout
+    /* empty layout */
+    if (!wlr_output)
       return false;
-    }
     output = PHOC_OUTPUT (wlr_output->data);
   }
 
@@ -1027,15 +1169,15 @@ view_center (PhocView *view, PhocOutput *output)
 
 
 static void
-phoc_view_init_subsurfaces (PhocView *view, struct wlr_surface *surface)
+phoc_view_init_subsurfaces (PhocView *self)
 {
   struct wlr_subsurface *subsurface;
 
-  wl_list_for_each (subsurface, &surface->current.subsurfaces_below, current.link)
-    phoc_subsurface_new (view, subsurface);
+  wl_list_for_each (subsurface, &self->wlr_surface->current.subsurfaces_below, current.link)
+    phoc_subsurface_new (PHOC_CHILD_ROOT (self), subsurface);
 
-  wl_list_for_each (subsurface, &surface->current.subsurfaces_above, current.link)
-    phoc_subsurface_new (view, subsurface);
+  wl_list_for_each (subsurface, &self->wlr_surface->current.subsurfaces_above, current.link)
+    phoc_subsurface_new (PHOC_CHILD_ROOT (self), subsurface);
 }
 
 
@@ -1046,7 +1188,7 @@ phoc_view_handle_surface_new_subsurface (struct wl_listener *listener, void *dat
   PhocView *self = PHOC_VIEW_SELF (priv);
   struct wlr_subsurface *wlr_subsurface = data;
 
-  phoc_subsurface_new (self, wlr_subsurface);
+  phoc_subsurface_new (PHOC_CHILD_ROOT (self), wlr_subsurface);
 }
 
 static gchar *
@@ -1124,7 +1266,7 @@ phoc_view_map (PhocView *self, struct wlr_surface *surface)
   g_assert (self->wlr_surface == NULL);
   self->wlr_surface = surface;
 
-  phoc_view_init_subsurfaces (self, self->wlr_surface);
+  phoc_view_init_subsurfaces (self);
   priv->surface_new_subsurface.notify = phoc_view_handle_surface_new_subsurface;
   wl_signal_add (&self->wlr_surface->events.new_subsurface, &priv->surface_new_subsurface);
 
@@ -1132,7 +1274,7 @@ phoc_view_map (PhocView *self, struct wlr_surface *surface)
     phoc_view_appear_activated (self, true);
 
     if (phoc_desktop_has_views (desktop)) {
-      // mapping a new stack may make the old stack disappear, so damage its area
+      /* Mapping a new stack may make the old stack disappear, so damage its area */
       PhocView *top_view = phoc_desktop_get_view_by_index (desktop, 0);
       while (top_view) {
         phoc_view_damage_whole (top_view);
@@ -1277,10 +1419,6 @@ phoc_view_setup (PhocView *view)
                                                  phoc_view_is_fullscreen (view));
   wlr_foreign_toplevel_handle_v1_set_maximized (priv->toplevel_handle,
                                                 phoc_view_is_maximized (view));
-  wlr_foreign_toplevel_handle_v1_set_title (priv->toplevel_handle,
-                                            priv->title ?: "");
-  wlr_foreign_toplevel_handle_v1_set_app_id (priv->toplevel_handle,
-                                             priv->app_id ?: "");
   if (view->parent)
     toplevel_handle = phoc_view_get_toplevel_handle (view->parent);
 
@@ -1373,12 +1511,20 @@ view_set_title (PhocView *view, const char *title)
 
   if (priv->toplevel_handle)
     wlr_foreign_toplevel_handle_v1_set_title (priv->toplevel_handle, title ?: "");
+
+  if (priv->ext_foreign_toplevel_v1_handle) {
+    struct wlr_ext_foreign_toplevel_handle_v1_state state = {
+      .app_id = priv->app_id,
+      .title = priv->title,
+    };
+    wlr_ext_foreign_toplevel_handle_v1_update_state (priv->ext_foreign_toplevel_v1_handle, &state);
+  }
 }
 
 void
 view_set_parent (PhocView *view, PhocView *parent)
 {
-  // setting a new parent may cause a cycle
+  /* Setting a new parent may cause a cycle */
   PhocView *node = parent;
   PhocViewPrivate *priv;
   struct wlr_foreign_toplevel_handle_v1 *toplevel_handle = NULL;
@@ -1444,6 +1590,14 @@ phoc_view_set_app_id (PhocView *view, const char *app_id)
 
   if (priv->toplevel_handle)
     wlr_foreign_toplevel_handle_v1_set_app_id (priv->toplevel_handle, app_id ?: "");
+
+  if (priv->ext_foreign_toplevel_v1_handle) {
+    struct wlr_ext_foreign_toplevel_handle_v1_state state = {
+      .app_id = priv->app_id,
+      .title = priv->title,
+    };
+    wlr_ext_foreign_toplevel_handle_v1_update_state (priv->ext_foreign_toplevel_v1_handle, &state);
+  }
 }
 
 
@@ -1551,7 +1705,7 @@ phoc_view_finalize (GObject *object)
   if (self->wlr_surface)
     phoc_view_unmap (self);
 
-  // Can happen if fullscreened while unmapped, and hasn't been mapped
+  /* Can happen if fullscreened while unmapped, and hasn't been mapped */
   if (phoc_view_is_fullscreen (self))
     priv->fullscreen_output->fullscreen_view = NULL;
 
@@ -1581,14 +1735,10 @@ phoc_view_for_each_surface_default (PhocView                    *self,
 static void
 phoc_view_get_geometry_default (PhocView *self, struct wlr_box *geom)
 {
-  PhocViewPrivate *priv;
-
-  priv = phoc_view_get_instance_private (self);
-
   geom->x = 0;
   geom->y = 0;
-  geom->width = self->box.width * priv->scale;
-  geom->height = self->box.height * priv->scale;
+  geom->width = self->box.width;
+  geom->height = self->box.height;
 }
 
 
@@ -1618,6 +1768,14 @@ phoc_view_get_wlr_surface_at_default (PhocView *self,
   return wlr_surface_surface_at (self->wlr_surface, sx, sy, sub_x, sub_y);
 }
 
+
+static float
+phoc_view_get_alpha_default (PhocView *self)
+{
+  return 1.0;
+}
+
+
 G_NORETURN
 static void
 phoc_view_resize_default (PhocView *self, uint32_t width, uint32_t height)
@@ -1685,6 +1843,7 @@ phoc_view_class_init (PhocViewClass *klass)
   view_class->set_tiled = phoc_view_set_tiled_default;
   view_class->set_suspended = phoc_view_set_suspended_default;
   view_class->get_wlr_surface_at = phoc_view_get_wlr_surface_at_default;
+  view_class->get_alpha = phoc_view_get_alpha_default;
   /* Mandatory */
   view_class->resize = phoc_view_resize_default;
   view_class->move_resize = phoc_view_move_resize_default;
@@ -1992,7 +2151,7 @@ phoc_view_get_alpha (PhocView *self)
   g_assert (PHOC_IS_VIEW (self));
   priv = phoc_view_get_instance_private (self);
 
-  return priv->alpha;
+  return priv->alpha * PHOC_VIEW_GET_CLASS (self)->get_alpha (self);
 }
 
 /**
@@ -2246,32 +2405,6 @@ phoc_view_arrange (PhocView *self, PhocOutput *output, gboolean center)
     view_center (self, output);
 }
 
-
-void
-phoc_view_add_child (PhocView *self, PhocViewChild *child)
-{
-  PhocViewPrivate *priv;
-
-  g_assert (PHOC_IS_VIEW (self));
-  g_assert (PHOC_IS_VIEW_CHILD (child));
-  priv = phoc_view_get_instance_private (self);
-
-  priv->child_surfaces = g_slist_prepend (priv->child_surfaces, child);
-}
-
-
-void
-phoc_view_remove_child (PhocView *self, PhocViewChild *child)
-{
-  PhocViewPrivate *priv;
-
-  g_assert (PHOC_IS_VIEW (self));
-  g_assert (PHOC_IS_VIEW_CHILD (child));
-  priv = phoc_view_get_instance_private (self);
-
-  priv->child_surfaces = g_slist_remove (priv->child_surfaces, child);
-}
-
 /**
  * phoc_view_set_always_on_top:
  * @self: a view
@@ -2333,59 +2466,3 @@ phoc_view_set_visibility (PhocView *self, gboolean visibility)
 
   phoc_view_set_suspended (self, !visibility);
 }
-
-/**
- * phoc_view_get_popup_unconstrain_region:
- * @self: The view
- * @box: (inout): The box
- *
- * Get the area to unconstrain a popup to relative to the parent
- * toplevels top left corner.
- *
- * Returns: Whether an unconstrain area was found
- */
-gboolean
-phoc_view_get_popup_unconstrain_region (PhocView *self, struct wlr_box *box)
-{
-  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
-  struct wlr_surface_output *surface_output;
-  struct wlr_box usable_area;
-  struct wlr_box geom;
-  PhocOutput *output;
-
-  g_assert (PHOC_IS_VIEW (self));
-
-  if (!phoc_view_is_mapped (self))
-    return FALSE;
-
-  /* Try top left corner of the view's geometry: */
-  phoc_view_get_geometry (self, &geom);
-  output = phoc_desktop_layout_get_output (desktop, self->box.x + geom.x, self->box.y + geom.y);
-  if (!output && wl_list_empty (&self->wlr_surface->current_outputs))
-    return FALSE;
-
-  /* Otherwise just take the first output */
-  surface_output = wl_container_of (self->wlr_surface->current_outputs.next, surface_output, link);
-  output = PHOC_OUTPUT (surface_output->output->data);
-  g_assert (PHOC_IS_OUTPUT (output));
-
-  if (!output) {
-    g_warning ("No output found for view %p at %d,%d", self, self->box.x, self->box.y);
-    return FALSE;
-  }
-
-  usable_area = output->usable_area;
-  usable_area.x += output->lx;
-  usable_area.y += output->ly;
-
-  /* the output box expressed in the coordinate system of the toplevel parent
-   * of the popup */
-  *box = (struct wlr_box) {
-    .x = usable_area.x - self->box.x,
-    .y = usable_area.y - self->box.y,
-    .width = usable_area.width,
-    .height = usable_area.height,
-  };
-
-  return TRUE;
-}
