@@ -1,6 +1,9 @@
 /*
  * Copyright (C) 2020 Purism SPC
+ *               2025 The Phosh Developers
+ *
  * SPDX-License-Identifier: GPL-3.0-or-later
+ *
  * Author: Guido Günther <agx@sigxcpu.org>
  */
 
@@ -13,10 +16,12 @@
 #include <sys/mman.h>
 
 struct task_data {
-  PhocTestClientFunc func;
-  gpointer data;
+  PhocTestClientFunc   func;
+  PhocTestOutputConfig output_config;
+  gpointer             data;
 };
 
+
 static bool
 abgr_to_argb (PhocTestBuffer *buffer)
 {
@@ -25,14 +30,14 @@ abgr_to_argb (PhocTestBuffer *buffer)
   guint8 *data = buffer->shm_data;
 
   for (int i = 0; i < buffer->height * buffer->stride; i += 4) {
-      guint32 *px = (guint32 *)(data + i);
-      guint8 r, g, b, a;
+    guint32 *px = (guint32 *)(data + i);
+    guint8 r, g, b, a;
 
-      a = (*px >> 24) & 0xFF;
-      b = (*px >> 16) & 0xFF;
-      g = (*px >> 8) & 0xFF;
-      r = *px & 0xFF;
-      *px = (a << 24) | (r << 16) | (g << 8) | b;
+    a = (*px >> 24) & 0xFF;
+    b = (*px >> 16) & 0xFF;
+    g = (*px >> 8) & 0xFF;
+    r = *px & 0xFF;
+    *px = (a << 24) | (r << 16) | (g << 8) | b;
   }
 
   switch (buffer->format) {
@@ -48,8 +53,9 @@ abgr_to_argb (PhocTestBuffer *buffer)
   return true;
 }
 
+
 static void
-buffer_to_argb(PhocTestBuffer *buffer)
+buffer_to_argb (PhocTestBuffer *buffer)
 {
   switch (buffer->format) {
   case WL_SHM_FORMAT_XRGB8888:
@@ -57,20 +63,21 @@ buffer_to_argb(PhocTestBuffer *buffer)
     break;
   case WL_SHM_FORMAT_XBGR8888:
   case WL_SHM_FORMAT_ABGR8888:
-    abgr_to_argb(buffer);
+    abgr_to_argb (buffer);
     break;
   default:
     g_assert_not_reached ();
   }
 }
 
+
 static void
-screencopy_frame_handle_buffer (void *data,
+screencopy_frame_handle_buffer (void                            *data,
                                 struct zwlr_screencopy_frame_v1 *handle,
-                                uint32_t format,
-                                uint32_t width,
-                                uint32_t height,
-                                uint32_t stride)
+                                uint32_t                         format,
+                                uint32_t                         width,
+                                uint32_t                         height,
+                                uint32_t                         stride)
 {
   PhocTestScreencopyFrame *frame = data;
   gboolean success;
@@ -81,17 +88,19 @@ screencopy_frame_handle_buffer (void *data,
                                                 height,
                                                 format);
   g_assert_true (success);
-  zwlr_screencopy_frame_v1_copy(handle, frame->buffer.wl_buffer);
+  zwlr_screencopy_frame_v1_copy (handle, frame->buffer.wl_buffer);
 }
 
+
 static void
-screencopy_frame_handle_flags(void *data, struct zwlr_screencopy_frame_v1 *handle, uint32_t flags)
+screencopy_frame_handle_flags (void *data, struct zwlr_screencopy_frame_v1 *handle, uint32_t flags)
 {
   PhocTestScreencopyFrame *frame = data;
 
   frame->flags = flags;
 }
 
+
 static void
 screencopy_frame_handle_ready (void *data, struct zwlr_screencopy_frame_v1 *handle,
                                uint32_t tv_sec_hi, uint32_t tv_sec_lo,
@@ -102,13 +111,15 @@ screencopy_frame_handle_ready (void *data, struct zwlr_screencopy_frame_v1 *hand
   frame->done = TRUE;
 }
 
+
 G_NORETURN
 static void
-screencopy_frame_handle_failed(void *data, struct zwlr_screencopy_frame_v1 *frame)
+screencopy_frame_handle_failed (void *data, struct zwlr_screencopy_frame_v1 *frame)
 {
   g_assert_not_reached ();
 }
 
+
 static const struct zwlr_screencopy_frame_v1_listener screencopy_frame_listener = {
   .buffer = screencopy_frame_handle_buffer,
   .flags = screencopy_frame_handle_flags,
@@ -116,6 +127,7 @@ static const struct zwlr_screencopy_frame_v1_listener screencopy_frame_listener
   .failed = screencopy_frame_handle_failed,
 };
 
+
 static void
 shm_format (void *data, struct wl_shm *wl_shm, guint32 format)
 {
@@ -124,56 +136,69 @@ shm_format (void *data, struct wl_shm *wl_shm, guint32 format)
   globals->formats |= (1 << format);
 }
 
+
 static void
 buffer_release (void *data, struct wl_buffer *buffer)
 {
   /* TBD */
 }
 
+
 struct wl_shm_listener shm_listener = {
   shm_format,
 };
 
+
 static const struct wl_buffer_listener buffer_listener = {
   buffer_release,
 };
 
+
 static void
 output_handle_geometry (void *data, struct wl_output *wl_output,
                         int32_t x, int32_t y, int32_t physical_width, int32_t physical_height,
                         int32_t subpixel, const char *make, const char *model,
-                        int32_t transform) {
+                        int32_t transform)
+{
   /* TBD */
 }
 
+
 static void
 output_handle_mode (void *data, struct wl_output *wl_output,
                     uint32_t flags, int32_t width, int32_t height, int32_t refresh)
 {
-  PhocTestOutput *output = data;
+  PhocTestClientGlobals *globals = data;
+  PhocTestOutputConfig output_config = globals->output_config;
 
   if ((flags & WL_OUTPUT_MODE_CURRENT) != 0) {
-    /* Make sure we got the right mode to not mess up screenshot comparisons */
-    g_assert_cmpint (width, ==, 1024);
-    g_assert_cmpint (height, ==, 768);
-    output->width = width;
-    output->height = height;
+    /* Output must have the configure mode */
+    g_assert_cmpint (width, ==, output_config.width);
+    g_assert_cmpint (height, ==, output_config.height);
+    globals->output.width = width;
+    globals->output.height = height;
   }
 }
 
+
 static void
 output_handle_done (void *data, struct wl_output *wl_output)
 {
   /* TBD */
 }
 
+
 static void
 output_handle_scale (void *data, struct wl_output *wl_output,
                      int32_t scale)
 {
-  g_assert_cmpint (scale, ==, 1);
+  PhocTestClientGlobals *globals = data;
+  PhocTestOutputConfig output_config = globals->output_config;
+
+  g_assert_cmpint(scale, ==, ceil(output_config.scale));
 }
 
+
 static const struct wl_output_listener output_listener = {
   .geometry = output_handle_geometry,
   .mode = output_handle_mode,
@@ -181,65 +206,72 @@ static const struct wl_output_listener output_listener = {
   .scale = output_handle_scale,
 };
 
+
 static void
-xdg_wm_base_ping(void *data, struct xdg_wm_base *shell, uint32_t serial)
+xdg_wm_base_ping (void *data, struct xdg_wm_base *shell, uint32_t serial)
 {
-  xdg_wm_base_pong(shell, serial);
+  xdg_wm_base_pong (shell, serial);
 }
 
+
 static const struct xdg_wm_base_listener wm_base_listener = {
   xdg_wm_base_ping,
 };
 
 
 static void
-foreign_toplevel_handle_title (void *data,
-                               struct zwlr_foreign_toplevel_handle_v1 *zwlr_foreign_toplevel_handle_v1,
-                               const char* title)
+foreign_toplevel_handle_title (void                                   *data,
+                               struct zwlr_foreign_toplevel_handle_v1 *handle,
+                               const char                             *title)
 {
   PhocTestForeignToplevel *toplevel = data;
   toplevel->title = g_strdup (title);
   g_debug ("Got toplevel's title: %p %s", toplevel, title);
 }
 
-static void
-foreign_toplevel_handle_app_id (void *data,
-                                struct zwlr_foreign_toplevel_handle_v1 *zwlr_foreign_toplevel_handle_v1,
-                                const char* app_id)
-{
-}
 
 static void
-foreign_toplevel_handle_output_enter (void *data,
-                                      struct zwlr_foreign_toplevel_handle_v1 *zwlr_foreign_toplevel_handle_v1,
-                                      struct wl_output *output)
-{
-}
-
-static void
-foreign_toplevel_handle_output_leave (void *data,
-                                      struct zwlr_foreign_toplevel_handle_v1 *zwlr_foreign_toplevel_handle_v1,
-                                      struct wl_output *output)
-{
-}
-
-static void
-foreign_toplevel_handle_state (void *data,
-                               struct zwlr_foreign_toplevel_handle_v1 *zwlr_foreign_toplevel_handle_v1,
-                               struct wl_array *state)
-{
-}
-
-static void
-foreign_toplevel_handle_done (void *data,
-                              struct zwlr_foreign_toplevel_handle_v1 *zwlr_foreign_toplevel_handle_v1)
+foreign_toplevel_handle_app_id (void                                   *data,
+                                struct zwlr_foreign_toplevel_handle_v1 *handle,
+                                const char                             *app_id)
 {
 }
 
 
 static void
-foreign_toplevel_handle_closed (void *data,
-                                struct zwlr_foreign_toplevel_handle_v1 *zwlr_foreign_toplevel_handle_v1)
+foreign_toplevel_handle_output_enter (void                                   *data,
+                                      struct zwlr_foreign_toplevel_handle_v1 *handle,
+                                      struct wl_output                       *output)
+{
+}
+
+
+static void
+foreign_toplevel_handle_output_leave (void                                   *data,
+                                      struct zwlr_foreign_toplevel_handle_v1 *handle,
+                                      struct wl_output                       *output)
+{
+}
+
+
+static void
+foreign_toplevel_handle_state (void                                   *data,
+                               struct zwlr_foreign_toplevel_handle_v1 *handle,
+                               struct wl_array                        *state)
+{
+}
+
+
+static void
+foreign_toplevel_handle_done (void                                   *data,
+                              struct zwlr_foreign_toplevel_handle_v1 *handle)
+{
+}
+
+
+static void
+foreign_toplevel_handle_closed (void                                   *data,
+                                struct zwlr_foreign_toplevel_handle_v1 *handle)
 {
   PhocTestForeignToplevel *toplevel = data;
   PhocTestClientGlobals *globals = toplevel->globals;
@@ -250,6 +282,7 @@ foreign_toplevel_handle_closed (void *data,
   g_free (toplevel);
 }
 
+
 static const struct zwlr_foreign_toplevel_handle_v1_listener foreign_toplevel_handle_listener = {
   .title = foreign_toplevel_handle_title,
   .app_id = foreign_toplevel_handle_app_id,
@@ -263,9 +296,9 @@ static const struct zwlr_foreign_toplevel_handle_v1_listener foreign_toplevel_ha
 
 
 static void
-foreign_toplevel_manager_handle_toplevel (void *data,
-                                          struct zwlr_foreign_toplevel_manager_v1 *zwlr_foreign_toplevel_manager_v1,
-                                          struct zwlr_foreign_toplevel_handle_v1 *handle)
+foreign_toplevel_manager_handle_toplevel (void                                    *data,
+                                          struct zwlr_foreign_toplevel_manager_v1 *manager,
+                                          struct zwlr_foreign_toplevel_handle_v1  *handle)
 {
   PhocTestClientGlobals *globals = data;
 
@@ -273,13 +306,15 @@ foreign_toplevel_manager_handle_toplevel (void *data,
   toplevel->handle = handle;
   toplevel->globals = globals;
   globals->foreign_toplevels = g_slist_append (globals->foreign_toplevels, toplevel);
-  zwlr_foreign_toplevel_handle_v1_add_listener (handle, &foreign_toplevel_handle_listener, toplevel);
+  zwlr_foreign_toplevel_handle_v1_add_listener (handle, &foreign_toplevel_handle_listener,
+                                                toplevel);
   g_debug ("New toplevel: %p", toplevel);
 }
 
+
 static void
-foreign_toplevel_manager_handle_finished (void *data,
-                                          struct zwlr_foreign_toplevel_manager_v1 *zwlr_foreign_toplevel_manager_v1)
+foreign_toplevel_manager_handle_finished (void                                    *data,
+                                          struct zwlr_foreign_toplevel_manager_v1 *handle)
 {
   g_debug ("wlr_foreign_toplevel_manager_finished");
 }
@@ -290,8 +325,13 @@ static const struct zwlr_foreign_toplevel_manager_v1_listener foreign_toplevel_m
   foreign_toplevel_manager_handle_finished
 };
 
-static void registry_handle_global(void *data, struct wl_registry *registry,
-                                   uint32_t name, const char *interface, uint32_t version)
+
+static void
+registry_handle_global (void               *data,
+                        struct wl_registry *registry,
+                        uint32_t            name,
+                        const char         *interface,
+                        uint32_t            version)
 {
   PhocTestClientGlobals *globals = data;
 
@@ -305,11 +345,11 @@ static void registry_handle_global(void *data, struct wl_registry *registry,
     g_assert_null (globals->output.output);
     globals->output.output = wl_registry_bind (registry, name,
                                                &wl_output_interface, 3);
-    wl_output_add_listener(globals->output.output, &output_listener, &globals->output);
+    wl_output_add_listener (globals->output.output, &output_listener, globals);
   } else if (!g_strcmp0 (interface, xdg_wm_base_interface.name)) {
     globals->xdg_shell = wl_registry_bind (registry, name,
-                                           &xdg_wm_base_interface, 1);
-    xdg_wm_base_add_listener(globals->xdg_shell, &wm_base_listener, NULL);
+                                           &xdg_wm_base_interface, 3);
+    xdg_wm_base_add_listener (globals->xdg_shell, &wm_base_listener, NULL);
   } else if (!g_strcmp0 (interface, zwlr_layer_shell_v1_interface.name)) {
     globals->layer_shell = wl_registry_bind (registry, name,
                                              &zwlr_layer_shell_v1_interface, 2);
@@ -318,7 +358,9 @@ static void registry_handle_global(void *data, struct wl_registry *registry,
                                                     &zwlr_screencopy_manager_v1_interface, 1);
   } else if (!g_strcmp0 (interface, zwlr_foreign_toplevel_manager_v1_interface.name)) {
     globals->foreign_toplevel_manager = wl_registry_bind (registry, name,
-                                                          &zwlr_foreign_toplevel_manager_v1_interface, 2);
+                                                          &
+                                                          zwlr_foreign_toplevel_manager_v1_interface,
+                                                          2);
     zwlr_foreign_toplevel_manager_v1_add_listener (globals->foreign_toplevel_manager,
                                                    &foreign_toplevel_manager_listener, globals);
   } else if (!g_strcmp0 (interface, phosh_private_interface.name)) {
@@ -330,35 +372,42 @@ static void registry_handle_global(void *data, struct wl_registry *registry,
                                                      &zphoc_layer_shell_effects_v1_interface, 3);
   } else if (!g_strcmp0 (interface, zxdg_decoration_manager_v1_interface.name)) {
     globals->decoration_manager = wl_registry_bind (registry, name,
-                                                     &zxdg_decoration_manager_v1_interface, 1);
+                                                    &zxdg_decoration_manager_v1_interface, 1);
   }
 }
 
-static void registry_handle_global_remove (void *data,
-                struct wl_registry *registry, uint32_t name) {
+
+static void
+registry_handle_global_remove (void               *data,
+                               struct wl_registry *registry,
+                               uint32_t            name)
+{
   // This space is intentionally left blank
 }
 
+
 static const struct wl_registry_listener registry_listener = {
   .global = registry_handle_global,
   .global_remove = registry_handle_global_remove,
 };
 
+
 static void
-wl_client_run (GTask *task, gpointer source,
-               gpointer data, GCancellable *cancel)
+wl_client_run (GTask *task, gpointer source, gpointer data, GCancellable *cancel)
 {
   struct wl_registry *registry;
   gboolean success = FALSE;
   struct task_data *td = data;
-  PhocTestClientGlobals globals = { 0 };
+  PhocTestClientGlobals globals = {
+    .output_config = td->output_config,
+    .display = wl_display_connect (NULL),
+  };
 
-  globals.display = wl_display_connect(NULL);
   g_assert_nonnull (globals.display);
-  registry = wl_display_get_registry(globals.display);
-  wl_registry_add_listener(registry, &registry_listener, &globals);
-  wl_display_dispatch(globals.display);
-  wl_display_roundtrip(globals.display);
+  registry = wl_display_get_registry (globals.display);
+  wl_registry_add_listener (registry, &registry_listener, &globals);
+  wl_display_dispatch (globals.display);
+  wl_display_roundtrip (globals.display);
   g_assert_nonnull (globals.compositor);
   g_assert_nonnull (globals.layer_shell);
   g_assert_nonnull (globals.shm);
@@ -391,12 +440,13 @@ wl_client_run (GTask *task, gpointer source,
   g_task_return_boolean (task, success);
 }
 
+
 static void
 on_wl_client_finish (GObject *source, GAsyncResult *res, gpointer data)
 {
   GMainLoop *loop = data;
   gboolean success;
-  g_autoptr(GError) err = NULL;
+  g_autoptr (GError) err = NULL;
 
   g_assert_true (g_task_is_valid (res, source));
   success = g_task_propagate_boolean (G_TASK (res), &err);
@@ -406,23 +456,92 @@ on_wl_client_finish (GObject *source, GAsyncResult *res, gpointer data)
   g_main_loop_quit (loop);
 }
 
-static gboolean
+
+G_NORETURN static void
 on_timer_expired (gpointer unused)
 {
   /* Compositor did not quit in time */
   g_assert_not_reached ();
-  return FALSE;
+}
+
+
+static PhocConfig *
+build_compositor_config (PhocTestClientIface *iface)
+{
+  PhocConfig *config;
+  PhocTestOutputConfig *output_config;
+  g_autofree char *config_str = NULL;
+  const char *transform;
+
+  g_assert (iface);
+  output_config = &iface->output_config;
+
+  if (!output_config->width)
+    output_config->width = 1024;
+  if (!output_config->height)
+    output_config->height = 768;
+
+  if (G_APPROX_VALUE (output_config->scale, 0.0, FLT_EPSILON))
+    output_config->scale = 1.0;
+
+  switch (output_config->transform) {
+  case WL_OUTPUT_TRANSFORM_NORMAL:
+    transform = "normal";
+    break;
+  case WL_OUTPUT_TRANSFORM_90:
+    transform = "90";
+    break;
+  case WL_OUTPUT_TRANSFORM_180:
+    transform = "180";
+    break;
+  case WL_OUTPUT_TRANSFORM_270:
+    transform = "270";
+    break;
+  case WL_OUTPUT_TRANSFORM_FLIPPED:
+    transform = "flipped";
+    break;
+  case WL_OUTPUT_TRANSFORM_FLIPPED_90:
+    transform = "flipped-90";
+    break;
+  case WL_OUTPUT_TRANSFORM_FLIPPED_180:
+    transform = "flipped-180";
+    break;
+  case WL_OUTPUT_TRANSFORM_FLIPPED_270:
+    transform = "flipped-270";
+    break;
+  default:
+    g_assert_not_reached ();
+  }
+
+  config_str = g_strdup_printf (
+    "[core]\n"
+    "xwayland=false\n"
+    "\n"
+    "[output:*%%*%%*]\n"
+    "mode=%dx%d\n"
+    "scale=%.2f\n"
+    "rotate=%s\n",
+    iface->output_config.width,
+    iface->output_config.height,
+    iface->output_config.scale,
+    transform);
+
+  config = phoc_config_new_from_data (config_str);
+  config->xwayland = iface->xwayland;
+
+  return config;
 }
 
 /**
  * phoc_test_client_run:
+ * @timeout: Abort test after timeout seconds
+ * @iface: Test client interface
+ * @data: Data passed verbatim to the test and prepare functions
  *
- * timeout: Abort test after timeout seconds
- * func: The test function to run
- * data: Data passed to the test function
+ * Run `func` from the test client iface in a Wayland client
+ * configured by the other parameters in the test client iface.
  *
- * Run func in a wayland client connected to compositor instance. The
- * test function is expected to return %TRUE on success and %FALSE
+ * The test function is expected to return %TRUE on success and %FALSE
  * otherwise.
  */
 void
@@ -430,29 +549,32 @@ phoc_test_client_run (gint timeout, PhocTestClientIface *iface, gpointer data)
 {
   PhocConfig *config;
   struct task_data td = { .data = data };
-  g_autoptr(PhocServer) server = phoc_server_get_default ();
-  g_autoptr(GMainLoop) loop = g_main_loop_new (NULL, FALSE);
-  g_autoptr(GTask) wl_client_task = g_task_new (NULL, NULL,
-                                                on_wl_client_finish,
-                                                loop);
-  if (iface)
-    td.func = iface->client_run;
+  g_autoptr (PhocServer) server = phoc_server_get_default ();
+  g_autoptr (GMainLoop) loop = g_main_loop_new (NULL, FALSE);
+  g_autoptr (GTask) wl_client_task = g_task_new (NULL, NULL, on_wl_client_finish, loop);
 
-  config = (iface && iface->config) ? iface->config : phoc_config_new_from_file (TEST_PHOC_INI);
+  config = build_compositor_config (iface);
+  if (!config)
+    config = phoc_config_new_from_file (TEST_PHOC_INI);
+
+  phoc_server_set_debug_flags (server, iface->debug_flags);
   g_assert_true (PHOC_IS_SERVER (server));
   g_assert_true (config);
-  g_assert_true (phoc_server_setup(server, config, NULL, loop,
-                                   PHOC_SERVER_FLAG_NONE,
-                                   iface ? iface->debug_flags : PHOC_SERVER_DEBUG_FLAG_NONE));
+  g_assert_true (phoc_server_setup (server, config, NULL, loop, PHOC_SERVER_FLAG_NONE));
   if (iface && iface->server_prepare)
-    g_assert_true (iface->server_prepare(server, data));
+    g_assert_true (iface->server_prepare (server, data));
 
+  if (iface) {
+    td.func = iface->client_run;
+    td.output_config = iface->output_config;
+  }
   g_task_set_task_data (wl_client_task, &td, NULL);
   g_task_run_in_thread (wl_client_task, wl_client_run);
-  g_timeout_add_seconds (timeout, on_timer_expired, NULL);
+  g_timeout_add_seconds_once (timeout, on_timer_expired, NULL);
   g_main_loop_run (loop);
 }
 
+
 static int
 create_anon_file (off_t size)
 {
@@ -460,15 +582,15 @@ create_anon_file (off_t size)
   int fd;
   int ret;
 
-  fd = mkstemp(template);
+  fd = mkstemp (template);
   g_assert_cmpint (fd, >=, 0);
 
   do {
     errno = 0;
-    ret = ftruncate(fd, size);
+    ret = ftruncate (fd, size);
   } while (errno == EINTR);
   g_assert_cmpint (ret, ==, 0);
-  unlink(template);
+  unlink (template);
   return fd;
 }
 
@@ -493,16 +615,16 @@ phoc_test_client_create_shm_buffer (PhocTestClientGlobals *globals,
   buffer->format = format;
   size = buffer->stride * height;
 
-  fd = create_anon_file(size);
+  fd = create_anon_file (size);
   g_assert_cmpint (fd, >=, 0);
 
   data = mmap (NULL, size, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
   g_assert (data != MAP_FAILED);
 
-  pool = wl_shm_create_pool(globals->shm, fd, size);
-  buffer->wl_buffer = wl_shm_pool_create_buffer(pool, 0,
-                                                width, height,
-                                                buffer->stride, format);
+  pool = wl_shm_create_pool (globals->shm, fd, size);
+  buffer->wl_buffer = wl_shm_pool_create_buffer (pool, 0,
+                                                 width, height,
+                                                 buffer->stride, format);
   wl_buffer_add_listener (buffer->wl_buffer, &buffer_listener, buffer);
   wl_shm_pool_destroy (pool);
   close (fd);
@@ -520,7 +642,6 @@ foreign_toplevel_compare (gconstpointer data, gconstpointer title)
 }
 
 /**
- *
  * phoc_test_client_get_foreign_toplevel_handle:
  *
  * Get the PhocTestForeignToplevel for a toplevel with the given title
@@ -530,7 +651,7 @@ foreign_toplevel_compare (gconstpointer data, gconstpointer title)
  */
 PhocTestForeignToplevel *
 phoc_test_client_get_foreign_toplevel_handle (PhocTestClientGlobals *globals,
-                                              const char *title)
+                                              const char            *title)
 {
   GSList *list = g_slist_find_custom (globals->foreign_toplevels, title, foreign_toplevel_compare);
   if (!list || !list->data)
@@ -539,9 +660,7 @@ phoc_test_client_get_foreign_toplevel_handle (PhocTestClientGlobals *globals,
   return list->data;
 }
 
-
 /**
- *
  * phoc_test_client_capture_frame:
  *
  * Capture the given wlr_screencopy_frame and return its screenshot buffer
@@ -549,14 +668,14 @@ phoc_test_client_get_foreign_toplevel_handle (PhocTestClientGlobals *globals,
  * Returns: (transfer none): The screenshot buffer.
  */
 PhocTestBuffer *
-phoc_test_client_capture_frame (PhocTestClientGlobals *globals,
-                                PhocTestScreencopyFrame *frame, struct zwlr_screencopy_frame_v1 *handle)
+phoc_test_client_capture_frame (PhocTestClientGlobals           *globals,
+                                PhocTestScreencopyFrame         *frame,
+                                struct zwlr_screencopy_frame_v1 *handle)
 {
   frame->globals = globals;
   g_assert_false (frame->done);
 
-  zwlr_screencopy_frame_v1_add_listener(handle,
-                                        &screencopy_frame_listener, frame);
+  zwlr_screencopy_frame_v1_add_listener (handle, &screencopy_frame_listener, frame);
   while (!frame->done && wl_display_dispatch (globals->display) != -1) {
   }
   g_assert_true (frame->done);
@@ -566,17 +685,18 @@ phoc_test_client_capture_frame (PhocTestClientGlobals *globals,
     guint32 height = frame->buffer.height;
     guint32 stride = frame->buffer.stride;
     guint8 *src = frame->buffer.shm_data;
-    g_autofree guint8 *dst = g_malloc0 (height * stride);
+    guint8 *dst = g_malloc0 (height * stride);
 
     for (guint i = 0, j = height - 1; i < height; i++, j--)
-      memmove((dst + (i * stride)), (src + (j * stride)), stride);
+      memcpy ((dst + (i * stride)), (src + (j * stride)), stride);
 
-    memmove (src, dst, height * stride);
+    memcpy (src, dst, height * stride);
     frame->flags &= ~ZWLR_SCREENCOPY_FRAME_V1_FLAGS_Y_INVERT;
-    /* There shouldn't be any other flags left */
-    g_assert_false (frame->flags);
+    g_free (dst);
   }
-  buffer_to_argb(&frame->buffer);
+  /* There shouldn't be any other flags left */
+  g_assert_false (frame->flags);
+  buffer_to_argb (&frame->buffer);
 
   frame->done = FALSE;
 
@@ -584,7 +704,6 @@ phoc_test_client_capture_frame (PhocTestClientGlobals *globals,
 }
 
 /**
- *
  * phoc_test_client_capture_output:
  *
  * Capture the given output and return its screenshot buffer
@@ -593,9 +712,10 @@ phoc_test_client_capture_frame (PhocTestClientGlobals *globals,
  */
 PhocTestBuffer *
 phoc_test_client_capture_output (PhocTestClientGlobals *globals,
-                                 PhocTestOutput *output)
+                                 PhocTestOutput        *output)
 {
-  struct zwlr_screencopy_frame_v1 *handle = zwlr_screencopy_manager_v1_capture_output (globals->screencopy_manager, FALSE, output->output);
+  struct zwlr_screencopy_frame_v1 *handle =
+    zwlr_screencopy_manager_v1_capture_output (globals->screencopy_manager, FALSE, output->output);
   phoc_test_client_capture_frame (globals, &output->screenshot, handle);
 
   g_assert_cmpint (output->screenshot.buffer.width, ==, output->width);
@@ -606,39 +726,37 @@ phoc_test_client_capture_output (PhocTestClientGlobals *globals,
 }
 
 /**
- *
  * phoc_test_buffer_equal:
  *
  * Compare two buffers
  *
  * Returns: %TRUE if buffers have identical content, otherwise %FALSE
  */
-gboolean phoc_test_buffer_equal (PhocTestBuffer *buf1, PhocTestBuffer *buf2)
+gboolean
+phoc_test_buffer_equal (PhocTestBuffer *buf1, PhocTestBuffer *buf2)
 {
   guint8 *c1 = buf1->shm_data;
   guint8 *c2 = buf2->shm_data;
 
   g_assert_true (buf1->format == WL_SHM_FORMAT_XRGB8888
-                || buf1->format == WL_SHM_FORMAT_ARGB8888);
+                 || buf1->format == WL_SHM_FORMAT_ARGB8888);
   g_assert_true (buf2->format == WL_SHM_FORMAT_XRGB8888
-                || buf2->format == WL_SHM_FORMAT_ARGB8888);
+                 || buf2->format == WL_SHM_FORMAT_ARGB8888);
 
-  if (buf1->width != buf2->width ||
-      buf1->height != buf2->height) {
+  if (buf1->width != buf2->width || buf1->height != buf2->height)
     return FALSE;
-  }
 
   for (guint y = 0; y < buf1->height; y++) {
     for (guint x = 0; x < buf1->width; x++) {
       // B
       if (c1[y * buf1->stride + x * 4 + 0] != c2[y * buf2->stride + x * 4 + 0])
-          return FALSE;
+        return FALSE;
       // G
       if (c1[y * buf1->stride + x * 4 + 1] != c2[y * buf2->stride + x * 4 + 1])
-          return FALSE;
+        return FALSE;
       // R
       if (c1[y * buf1->stride + x * 4 + 2] != c2[y * buf2->stride + x * 4 + 2])
-          return FALSE;
+        return FALSE;
       // A/X
       if (buf1->format == WL_SHM_FORMAT_ARGB8888 && buf2->format == WL_SHM_FORMAT_ARGB8888)
         if (c1[y * buf1->stride + x * 4 + 3] != c2[y * buf2->stride + x * 4 + 3])
@@ -683,7 +801,7 @@ phoc_test_buffer_save (PhocTestBuffer *buffer, const gchar *filename)
   g_assert_cmpint (status, ==, CAIRO_STATUS_SUCCESS);
   g_debug ("Saved buffer png %s", filename);
 
-  cairo_surface_destroy(surface);
+  cairo_surface_destroy (surface);
   return TRUE;
 }
 
@@ -703,24 +821,24 @@ phoc_test_buffer_matches_screenshot (PhocTestBuffer *buffer, const gchar *filena
                  || buffer->format == WL_SHM_FORMAT_ARGB8888);
 
   switch (cairo_surface_status (surface)) {
-    case CAIRO_STATUS_NO_MEMORY:
-      msg = "no memory";
-      break;
-    case CAIRO_STATUS_FILE_NOT_FOUND:
-      msg = "file not found";
-      break;
-    case CAIRO_STATUS_READ_ERROR:
-      msg = "read error";
-      break;
-    case CAIRO_STATUS_PNG_ERROR:
-      msg = "png error";
-      break;
-    default:
-      msg = NULL;
+  case CAIRO_STATUS_NO_MEMORY:
+    msg = "no memory";
+    break;
+  case CAIRO_STATUS_FILE_NOT_FOUND:
+    msg = "file not found";
+    break;
+  case CAIRO_STATUS_READ_ERROR:
+    msg = "read error";
+    break;
+  case CAIRO_STATUS_PNG_ERROR:
+    msg = "png error";
+    break;
+  default:
+    msg = NULL;
   }
 
   if (msg)
-    g_error("Failed to load screenshot %s: %s", filename, msg);
+    g_error ("Failed to load screenshot %s: %s", filename, msg);
 
   format = cairo_image_surface_get_format (surface);
   switch (format) {
@@ -731,7 +849,7 @@ phoc_test_buffer_matches_screenshot (PhocTestBuffer *buffer, const gchar *filena
     mask = 0xFFFFFFFF;
     break;
   default:
-    g_assert_not_reached();
+    g_assert_not_reached ();
   }
 
   if (buffer->height != cairo_image_surface_get_height (surface) ||
@@ -772,9 +890,9 @@ phoc_test_buffer_free (PhocTestBuffer *buffer)
 #define DEFAULT_HEIGHT 200
 
 static void
-xdg_toplevel_handle_configure(void *data, struct xdg_toplevel *xdg_toplevel,
-                              int32_t width, int32_t height,
-                              struct wl_array *states)
+xdg_toplevel_handle_configure (void *data, struct xdg_toplevel *xdg_toplevel,
+                               int32_t width, int32_t height,
+                               struct wl_array *states)
 {
   PhocTestXdgToplevelSurface *xs = data;
 
@@ -787,13 +905,13 @@ xdg_toplevel_handle_configure(void *data, struct xdg_toplevel *xdg_toplevel,
 }
 
 static void
-xdg_surface_handle_configure(void *data, struct xdg_surface *xdg_surface,
-                             uint32_t serial)
+xdg_surface_handle_configure (void *data, struct xdg_surface *xdg_surface,
+                              uint32_t serial)
 {
   PhocTestXdgToplevelSurface *xs = data;
 
   g_debug ("Configured %p serial %d", xdg_surface, serial);
-  xdg_surface_ack_configure(xs->xdg_surface, serial);
+  xdg_surface_ack_configure (xs->xdg_surface, serial);
   xs->configured = TRUE;
 }
 
@@ -802,7 +920,7 @@ static const struct xdg_surface_listener xdg_surface_listener = {
 };
 
 static void
-xdg_toplevel_handle_close(void *data, struct xdg_toplevel *xdg_surface)
+xdg_toplevel_handle_close (void *data, struct xdg_toplevel *xdg_surface)
 {
   /* TBD */
 }
@@ -812,7 +930,6 @@ static const struct xdg_toplevel_listener xdg_toplevel_listener = {
   xdg_toplevel_handle_close,
 };
 
-
 /**
  * phoc_test_xdg_toplevel_new:
  * @globals: The wayland globals
@@ -823,6 +940,8 @@ static const struct xdg_toplevel_listener xdg_toplevel_listener = {
  * Creates a xdg toplevel with the given property for use in tests.
  * If width and/or height are 0, defaults will be used. Free with
  * `phoc_test_xdg_toplevel_free`.
+ *
+ * Returns: The toplevel surface
  */
 PhocTestXdgToplevelSurface *
 phoc_test_xdg_toplevel_new (PhocTestClientGlobals *globals,
@@ -835,8 +954,7 @@ phoc_test_xdg_toplevel_new (PhocTestClientGlobals *globals,
   xs->wl_surface = wl_compositor_create_surface (globals->compositor);
   g_assert_nonnull (xs->wl_surface);
 
-  xs->xdg_surface = xdg_wm_base_get_xdg_surface (globals->xdg_shell,
-                                                 xs->wl_surface);
+  xs->xdg_surface = xdg_wm_base_get_xdg_surface (globals->xdg_shell, xs->wl_surface);
   g_assert_nonnull (xs->wl_surface);
   xdg_surface_add_listener (xs->xdg_surface,
                             &xdg_surface_listener, xs);
@@ -873,6 +991,8 @@ phoc_test_xdg_toplevel_new (PhocTestClientGlobals *globals,
  * `phoc_test_xdg_toplevel_free`.
  *
  * This functions also attaches a buffer with the given color.
+ *
+ * Returns: The toplevel surface
  */
 PhocTestXdgToplevelSurface *
 phoc_test_xdg_toplevel_new_with_buffer (PhocTestClientGlobals *globals,
@@ -888,7 +1008,7 @@ phoc_test_xdg_toplevel_new_with_buffer (PhocTestClientGlobals *globals,
   phoc_test_client_create_shm_buffer (globals, &xs->buffer, xs->width, xs->height,
                                       WL_SHM_FORMAT_XRGB8888);
 
-  for (int i = 0; i < xs->width * xs->height * 4; i+=4)
+  for (int i = 0; i < xs->width * xs->height * 4; i += 4)
     *(guint32*)(xs->buffer.shm_data + i) = color;
 
   wl_surface_attach (xs->wl_surface, xs->buffer.wl_buffer, 0, 0);
@@ -927,7 +1047,7 @@ phoc_test_xdg_update_buffer (PhocTestClientGlobals      *globals,
   phoc_test_client_create_shm_buffer (globals, &buffer, xs->width, xs->height,
                                       WL_SHM_FORMAT_XRGB8888);
 
-  for (int i = 0; i < xs->width * xs->height * 4; i+=4)
+  for (int i = 0; i < xs->width * xs->height * 4; i += 4)
     *(guint32*)(buffer.shm_data + i) = color;
 
   wl_surface_attach (xs->wl_surface, buffer.wl_buffer, 0, 0);
@@ -956,7 +1076,7 @@ phoc_test_setup (PhocTestFixture *fixture, gconstpointer data)
 
   fixture->bus = g_test_dbus_new (G_TEST_DBUS_NONE);
 
-  /* Preserve x11 display for xvfb-run */
+  /* Preserve X11 display for xvfb-run */
   display = g_getenv ("DISPLAY");
 
   g_test_dbus_up (fixture->bus);
@@ -1015,7 +1135,7 @@ phoc_test_teardown (PhocTestFixture *fixture, gconstpointer unused)
 
   g_autoptr (GFile) file = g_file_new_for_path (fixture->tmpdir);
 
-  /* Preserve x11 display for xvfb-run */
+  /* Preserve X11 display for xvfb-run */
   display = g_getenv ("DISPLAY");
 
   g_test_dbus_down (fixture->bus);
